#include "broadphase.h"
#include "world.h"
#include "body.h"
#include "aarect.h"

#include <boost/foreach.hpp>

using namespace physics;

template <typename d>
typename d::Body* physics::BroadPhase<d>::findBeneath (const Vector& p) {
	typedef typename World::body_container::iterator iterator;
	
	BOOST_FOREACH(Body* body, m_world->bodies) {
		if (body->contains(p)) {
			return body;
		}
	}
	
	return 0;
}

template <typename d>
typename d::Body* physics::BroadPhase<d>::findBeneath (const AARect& rect) {
	Body* bodyFound = 0;

	//TODO: Implement

	return bodyFound;
}

template <typename d>
typename std::list<typename d::Body*> physics::BroadPhase<d>::findBeneathAll (const Vector& position) {
	//List of bodies to return
	std::list<Body*> bodies;

	//TODO Implement

	return bodies;
}

template <typename d>
typename std::list<typename d::Body*> physics::BroadPhase<d>::findBeneathAll (const AARect& r) {
	//List of bodies to return
	std::list<Body*> bodies;

	BOOST_FOREACH(Body* body, m_world->bodies) {
		if (body->intersects(r)) {
			bodies.push_back(body);
		}
	}
	
	return bodies;
}

template <typename d>
typename d::Body* physics::BroadPhase<d>::intersectRay (const Vector& start, const Vector& dir, Vector& intersectionPoint) {
	float t = -1;
	Body* bodyFound = 0;
	
	BOOST_FOREACH(Body* b, m_world->bodies) {
		Vector q;
		
		if (b->intersectsRay(start, dir, q)) {
			float distance = (q - start).dot(dir);
			
			if (distance < t || t < 0) {
				t = distance;
				intersectionPoint = q;
				bodyFound = b;
			}
		}
	}
	
	return bodyFound;
}

template <typename d>
typename physics::BroadPhase<d>::output_container physics::BroadPhase<d>::findPairs () {
	typedef typename World::body_container::iterator iterator;
	
	output_container pairs;
	
	for (iterator i = m_world->bodies.begin(); i != m_world->bodies.end(); i++) {
		Body* body1 = *i;
		
		// init j to be the next element after i
		iterator j = i;
		j++;
		
		for (; j != m_world->bodies.end(); j++) {
			Body* body2 = *j;
			
			if (test(body1, body2)) {
				pairs.push_back(BodyPair(body1, body2));
			}
		}
	}
	
	return pairs;
}

// Because we're implementing methods of a template class in a cpp file
// (rather than in the header), the compiler can have trouble linking the
// declaration and the implementation.
// To fix this, we use the following magic line of code.
// See: http://www.parashift.com/c++-faq-lite/templates.html#faq-35.15
// Note that this _must_ be at the bottom of the file

template class BroadPhase<TwoDee>;
template class BroadPhase<ThreeDee>;

