#ifndef _BROADPHASE_QUADTREE_H_
#define _BROADPHASE_QUADTREE_H_

#include "common.h"
#include "broadphase.h"

namespace physics {
		/*!
	*	Broadphase implementation that uses Quadtree spacial hashing
	*	to efficiently check the proximity of bodies within the 2D
	*	simulation.
	*	Performant for large numbers of bodies in one simulation
	*/
	class QuadtreeBroadPhase : public BroadPhase<TwoDee> {
		public:
		
		USE_2D();
		
		QuadtreeBroadPhase () {}
		
		virtual void init (World* world);
		
		virtual output_container findPairs ();
		
		virtual void bodyAdded (Body* body);
		virtual void bodyRemoved (Body* body);
				
		private:
		
		struct Node {
			Vector centre;
			int key; // Morton key of a node.
			int depth;
			unsigned char hasChild;
			std::list<Body*> bodies;
		};
		
		struct Quadtree { 
			std::map<int, Node*> nodes;
			int width;
			int height;
			int levels;
		};
		
		// Octree struct.
		Quadtree* tree;
		
		// Map of bodies to locations codes.
		std::map<Body*, unsigned int> locations;
		
		// Morton key calculations
		unsigned int generateMortonKey (const Vector& vector);
		unsigned int truncate (int depth, unsigned int key);
		int calcDepth (unsigned int key);
		
		// Recursive functions for iterating over the tree.
		unsigned insert (Node* node, Body* body, const Vector &low, const Vector &high, int depth=0);
		void remove (Body* body);
		void buildPairs (Node* node, output_container& list);
	};
};

#endif

