#ifndef _PHYSICS_COLLISION_LISTENER_
#define _PHYSICS_COLLISION_LISTENER_

#include "common.h"

namespace physics {
	template <typename d> class Contact;
	
	/**
	 * Description of what a CollisionListener is and how it should be used goes here.
	 * Note: CollisionListeners are never deleted by Large Polygon Collider: that is the user's responsibility.
	 */
	template <typename d>
	/*! 
	*	Class that implements a collision listener. Provides call backs so that events within the physics simulation
	*	can be acted on by external applications.
	*/
	
	class CollisionListener {
		public:
		//! Constructor
		CollisionListener () {}
		//! Destructor
		virtual ~CollisionListener () {}
		
		
		//! Notifies when a contact has been detected
		/*! \param contact Pointer to a contact
		* 	\param index Integer; 0 or 1 depending on which body registered the listener
		*	\return Return false to ignore contact, Return true to process contact
		*/ 
		virtual bool notify (Contact<d>* contact, int index) = 0;
	};
};

#endif

