#ifndef _PHYSICS_CONTACT_
#define _PHYSICS_CONTACT_

#include "common.h"

#include "vector.h"

#include <vector>

// TODO: sort out what member data this class needs

namespace physics {
	
	template <typename d>
	/*!
	*	This class contains the definition of a contact between two bodies in the simulation
	*/
	class Contact
	{
		public:
		
		USE_DIMENSION(d);
		
		union {
			Body* body[2];
			
			struct {
				Body* body0;
				Body* body1;
			};
		};
		
		union {
			Shape* shape[2];
			
			struct {
				Shape* shape0;
				Shape* shape1;
			};
		};
		
		//! Normal in World coordinates
		Vector normal;
		//! Position in World coordinates
		Vector position;
		
		float penetration;
		float approachSpeed;
		//! Accumulated normal impulse
		float Pn;
		//! Accumulated normal bias impulse
		float Pnb;	
		//! Accumulated friction impulse
		Vector frictionImpulse;
		
		float massNormal;
		float bias;
		
		//! Relative velocity in contact coordinates
		Vector relativeVelocity;
		
		Vector offset[2];
		
		float inverseMass[2];
		
		// My gut feeling is that we shouldn't need both of these
		// They are in contact coordinates
		Matrix impulsePerVelocity;
		Matrix velocityPerImpulse;
		
		Matrix contactToWorld;
		
		bool isEdge[2];
		//! Feature ID; Either an edgeID or vertexID (in 2D)
		int featureID[2];
		
		Contact () : penetration(), approachSpeed(), Pn(), Pnb(), massNormal(), bias() {}
		
		void init (Shape* a1, Shape* a2);
		//void recalculate ();
		bool invalid ();
		void calculateVelocities ();
		void calculatePenetration ();
		void calculateMass ();
		
		Vector getRelativeContactVelocity ();

		// TODO: remove magic number
		void setPenetration(float newPenetration) {
			if (newPenetration < penetration && newPenetration < -0.01) {
				body0->setAwake(true);
				body1->setAwake(true);
			}
			penetration = newPenetration;
		}
		
		bool operator < (const Contact<d>& c);
		bool operator == (const Contact<d>& c);
		bool operator > (const Contact<d>& c);
		
		bool operator <= (const Contact<d>& c) { return ! (*this > c); }
		bool operator != (const Contact<d>& c) { return ! (*this == c); }
		bool operator >= (const Contact<d>& c) { return ! (*this < c); }
		
		
	};
};

template <typename d>
inline bool physics::Contact<d>::operator == (const Contact& c) {
	if (shape[0] != c.shape[0]) return false;
	if (shape[1] != c.shape[1]) return false;
	
	if (isEdge[0] != c.isEdge[0]) return false;
	if (isEdge[1] != c.isEdge[1]) return false;
	
	if (featureID[0] != c.featureID[0]) return false;
	if (featureID[1] != c.featureID[1]) return false;
	
	return true;
}

template <typename d>
inline bool physics::Contact<d>::operator < (const Contact& c) {
	if (shape[0] < c.shape[0]) return true;
	if (shape[1] < c.shape[1]) return true;
	
	if (isEdge[0] < c.isEdge[0]) return true;
	if (isEdge[1] < c.isEdge[1]) return true;
	
	if (featureID[0] < c.featureID[0]) return true;
	if (featureID[1] < c.featureID[1]) return true;
	
	return false;
}

template <typename d>
inline bool physics::Contact<d>::operator > (const Contact& c) {
	if (shape[0] > c.shape[0]) return true;
	if (shape[1] > c.shape[1]) return true;
	
	if (isEdge[0] > c.isEdge[0]) return true;
	if (isEdge[1] > c.isEdge[1]) return true;
	
	if (featureID[0] > c.featureID[0]) return true;
	if (featureID[1] > c.featureID[1]) return true;
	
	return false;
}

#endif

