#ifndef _PHYSICS_CONTACT_GENERATOR_
#define _PHYSICS_CONTACT_GENERATOR_

#include "common.h"
#include "world.h"

#include <list>

namespace physics {
	
	class Box;
	class Sphere;
	
	template <typename d> class Contact;
	
	template <typename d>
	/*!
	*	Builds a list of contacts within the simulation
	*/
	class ContactGenerator {
		public:
		
		USE_DIMENSION(d);
		
		typedef physics::Contact<d> Contact;
		typedef std::pair<Body*, Body*> BodyPair;
		typedef physics::Joint<d> Joint;
		
		typedef typename World::body_pair_container input_container;
		typedef typename World::contact_container output_container;
		
		//! Constructor
		ContactGenerator () {}
		//! Destructor
		~ContactGenerator () {}
		
		void init (World* world) {}
		
		/*! Finds the contacts within the simulation
		*	\param input_container List of bodies, abstracted to allow for 2D/3D generalisation
		*	\return output_container List of contacts, abstracted to allow for 2D/3D generalisation
		*/
		output_container findContacts (const input_container&);
		
		void check (Body* a, Body* b);
		bool check (Shape* a, Shape* b);
		
		// quick hack
		output_container m_contacts;

		private:
		
		void checkAdjacent (Contact& c, int shape, int feature);
		
		// This is only relevant in 3D
		bool checkSphereSphere (Sphere* shape1, Sphere* shape2);
		bool checkBoxSphere (Box* shape1, Sphere* shape2, bool flip);
		bool checkSphereBox (Sphere* shape1, Box* shape2) { return checkBoxSphere(shape2, shape1, true); }
		bool checkBoxSphere (Box* shape1, Sphere* shape2) { return checkBoxSphere(shape1, shape2, false); }
		bool checkBoxBox (Box* shape1, Box* shape2);
		
		
	};
	
	// Declare that we're specialising this for 2D and 3D
	template <> bool ContactGenerator<TwoDee>::check(Shape* a, Shape* b);
	template <> bool ContactGenerator<ThreeDee>::check(Shape* a, Shape* b);
	
	template <> bool ContactGenerator<TwoDee>::checkSphereSphere (Sphere* shape1, Sphere* shape2);
	template <> bool ContactGenerator<TwoDee>::checkBoxSphere (Box* shape1, Sphere* shape2, bool flip);
	template <> bool ContactGenerator<TwoDee>::checkBoxBox (Box* shape1, Box* shape2);
	
	template <> bool ContactGenerator<ThreeDee>::checkSphereSphere (Sphere* shape1, Sphere* shape2);
	template <> bool ContactGenerator<ThreeDee>::checkBoxSphere (Box* shape1, Sphere* shape2, bool flip);
	template <> bool ContactGenerator<ThreeDee>::checkBoxBox (Box* shape1, Box* shape2);
};

#endif

