#ifndef _PHYSICS_CONTACT_LIST_
#define _PHYSICS_CONTACT_LIST_

#include "common.h"
#include "world.h"
#include "contact.h"

#include <list>

namespace physics {
	
	template <typename d> class Contact;
	
	template <typename d>
	class ContactList {
		public:
		
		typedef physics::Contact<d> Contact;
		typedef void(*callback_type)(Contact*);
		
		typedef typename physics::World<d>::contact_container input_container;
		
		ContactList () {}
		virtual ~ContactList () {}
		
		virtual void init (World<d>* w) {}
		
		virtual void addContacts (const input_container& newContacts) = 0;
		
		virtual unsigned int size () const = 0;
		
		virtual void removeContacts (Body<d>* body) = 0;
		
		virtual void foreach (callback_type callback) = 0;
		
		virtual void notifyListeners () = 0;
		
		virtual void prepareVelocities () = 0;
		virtual void preparePenetrations () = 0;
		
		virtual Contact* getGreatestVelocity () const = 0;
		virtual Contact* getGreatestPenetration () const = 0;
		
		virtual void updateVelocities (Contact* data) = 0;
		virtual void updatePenetrations (Contact* data, float* moveLinear) = 0;
	};
	
	/**
	* Order contacts so least element is that with greatest penetration.
	*/
	template <typename Contact>
	struct ComparePenetration {
		public:
		
		bool operator() (const Contact* a, const Contact* b)
		{
			return (a->penetration > b->penetration);
		}
		
		bool operator() (const Contact& a, const Contact& b)
		{
			return (a.penetration > b.penetration);
		}
	};

	/**
	* Order contacts so least element is that with greatest approach velocity
	*/
	template <typename Contact>
	struct CompareSpeed {
	public:
		
		bool operator() (const Contact* a, const Contact* b)
		{
			/*if (a->penetration < 0) return false;
			if (b->penetration < 0) return true;*/
			return (a->approachSpeed > b->approachSpeed);
		}
		
		bool operator() (const Contact& a, const Contact& b)
		{
			/*if (a.penetration < 0) return false;
			if (b.penetration < 0) return true;*/
			return (a.approachSpeed > b.approachSpeed);
		}
	};
};

#endif

