#ifndef _PHYSICS_CONTACT_LIST_BASE_
#define _PHYSICS_CONTACT_LIST_BASE_

#include "common.h"

#include "contact_list.h"

#include <vector>

namespace physics {
	
	template <typename d, typename C>
	class BaseContactList : public ContactList<d> {
		public:
		
		typedef physics::Contact<d> Contact;
		typedef void(*callback_type)(Contact*);
		typedef C container;
		
		typedef typename physics::World<d>::contact_container input_container;
		
		container contacts;
		
		BaseContactList (bool doCache = Globals::cacheContacts) : cacheContacts(doCache) {}
		~BaseContactList ();
		
		void add (Contact* c);
		
		Contact* find (Contact* c);
		
		virtual void addContacts (const input_container& newContacts);
		
		virtual unsigned int size () const { return (unsigned int) contacts.size(); }
		
		virtual void removeContacts (Body<d>* body);
		
		virtual void foreach (callback_type callback);
		
		virtual void notifyListeners ();
		
		virtual void prepareVelocities ();
		virtual void preparePenetrations ();
		
		virtual Contact* getGreatestVelocity () const;
		virtual Contact* getGreatestPenetration () const;
		
		virtual void updateVelocities (Contact* data);
		virtual void updatePenetrations (Contact* data, float* moveLinear);
		
		private:
		
		bool cacheContacts;
	};
	
	template <typename d, template <typename, typename> class C = std::vector >
	class NaiveContactList : public BaseContactList<d, C<Contact<d>*, std::allocator<Contact<d>*> > > {};
};

#endif

