#include "contact_list_multi_index.h"

#include <algorithm>
#include <boost/foreach.hpp>

using namespace physics;
using namespace std;

#if MULTI_INDEX_CONTACT_LIST_SORTED

template <typename d>
Contact<d>* MultiIndexContactList<d>::getGreatestVelocity () const {
	return *(Super::contacts.get<by_velocity>().begin());
}

template <typename d>
Contact<d>* MultiIndexContactList<d>::getGreatestPenetration () const {
	return *(Super::contacts.get<by_penetration>().begin());
}

#endif

template <typename d>
void MultiIndexContactList<d>::prepareVelocities () {
	for (iterator it = Super::contacts.begin(); it != Super::contacts.end(); it++) {
		Contact* c = *it;
		c->calculateVelocities();
		#if MULTI_INDEX_CONTACT_LIST_SORTED
		Super::contacts.replace(it, c);
		#endif
	}
}

template <typename d>
void MultiIndexContactList<d>::preparePenetrations () {
	#if MULTI_INDEX_CONTACT_LIST_SORTED
	for (iterator it = Super::contacts.begin(); it != Super::contacts.end(); it++) {
		Contact* c = *it;
		Super::contacts.replace(it, c);
	}
	#endif
}

template <typename d>
void MultiIndexContactList<d>::updateVelocities (Contact* c1)
{
	#if BODY_STORES_CONTACTS
	#if MULTI_INDEX_CONTACT_LIST_SORTED
		typedef typename container::template index<by_contact>::type index_type;
		typedef typename index_type::iterator index_iter;
		
		index_type& lookup = Super::contacts.get<by_contact>();
	#endif
	for (int i = 0; i < 2; i++) {
		Body<d>* body = c1->body[i];
		
		BOOST_FOREACH(Contact* c2, body->contacts) {
			c2->calculateVelocities();
			#if MULTI_INDEX_CONTACT_LIST_SORTED
				lookup.replace(lookup.find(c2), c2);
			#endif
		}
	}
	#else
	typedef typename container::template index<by_body_0>::type index0_type;
	typedef typename index0_type::iterator index0_iter;
	
	index0_type& index0 = Super::contacts.get<by_body_0>();
	
	for (int i = 0; i < 2; i++) {
		std::pair<index0_iter, index0_iter> bounds = index0.equal_range(c1->body[i]);
		
		for (index0_iter it = bounds.first; it != bounds.second; it++) {
			(**it).calculateVelocities();
			
			#if MULTI_INDEX_CONTACT_LIST_SORTED
			index0.replace(it, *it);
			#endif
		}
	}

	typedef typename container::template index<by_body_1>::type index1_type;
	typedef typename index1_type::iterator index1_iter;
	
	index1_type& index1 = Super::contacts.get<by_body_1>();
	
	for (int i = 0; i < 2; i++) {
		std::pair<index1_iter, index1_iter> bounds = index1.equal_range(c1->body[i]);
		
		for (index1_iter it = bounds.first; it != bounds.second; it++) {
			(**it).calculateVelocities();
			
			#if MULTI_INDEX_CONTACT_LIST_SORTED
			index1.replace(it, *it);
			#endif
		}
	}
	#endif
}

template <typename d>
void MultiIndexContactList<d>::updatePenetrations (Contact* c1, float* moveLinear)
{
	#if BODY_STORES_CONTACTS
	#if MULTI_INDEX_CONTACT_LIST_SORTED
		typedef typename container::template index<by_contact>::type index_type;
		typedef typename index_type::iterator index_iter;
		
		index_type& lookup = Super::contacts.get<by_contact>();
	#endif
	for (int i = 0; i < 2; i++) {
		if (moveLinear[i] == 0) { continue; }
		
		Body<d>* body = c1->body[i];
		
		BOOST_FOREACH(Contact* c2, body->contacts) {
			if (body == c2->body[0])
			{
				c2->penetration += moveLinear[i] * c1->normal.dot(c2->normal);
			}
			else// if (body == c2->body[1])
			{
				c2->penetration -= moveLinear[i] * c1->normal.dot(c2->normal);
			}
			
			#if MULTI_INDEX_CONTACT_LIST_SORTED
				lookup.replace(lookup.find(c2), c2);
			#endif
		}
	}
	#else
	typedef typename container::template index<by_body_0>::type index0_type;
	typedef typename index0_type::iterator index0_iter;
	
	index0_type& index0 = Super::contacts.get<by_body_0>();
	
	for (int i = 0; i < 2; i++) {
		std::pair<index0_iter, index0_iter> bounds = index0.equal_range(c1->body[i]);
		
		for (index0_iter it = bounds.first; it != bounds.second; it++) {
			Contact& c2 = **it;
			
			c2.penetration += moveLinear[i] * c1->normal.dot(c2.normal);
			
			#if MULTI_INDEX_CONTACT_LIST_SORTED
			index0.replace(it, *it);
			#endif
		}
	}

	typedef typename container::template index<by_body_1>::type index1_type;
	typedef typename index1_type::iterator index1_iter;
	
	index1_type& index1 = Super::contacts.get<by_body_1>();
	
	for (int i = 0; i < 2; i++) {
		std::pair<index1_iter, index1_iter> bounds = index1.equal_range(c1->body[i]);
		
		for (index1_iter it = bounds.first; it != bounds.second; it++) {
			Contact& c2 = **it;
			
			c2.penetration -= moveLinear[i] * c1->normal.dot(c2.normal);
			
			#if MULTI_INDEX_CONTACT_LIST_SORTED
			index1.replace(it, *it);
			#endif
		}
	}
	#endif
}

#include "contact_list_base.cpp"

template class MultiIndexContactList<TwoDee>;
template class MultiIndexContactList<ThreeDee>;

