#ifndef _PHYSICS_CONTACT_LIST_MULTI_INDEX_
#define _PHYSICS_CONTACT_LIST_MULTI_INDEX_

#include "common.h"

#include "contact_list_base.h"

// Macro horrible-ness in this file...

/** sequenced or random_access: Index type used to iterate over all contacts. */
#define MULTI_INDEX_CONTACT_LIST_DEFAULT_INDEX sequenced

/** 1 or 0: determines whether there is a sorted index to find greatest penetration/velocity. */
#define MULTI_INDEX_CONTACT_LIST_SORTED 0

/** ordered_non_unique or hashed_non_unique: Index type used to find all contacts for a given body. */
#define MULTI_INDEX_CONTACT_LIST_LOOKUP hashed_non_unique

#include <boost/multi_index_container.hpp>
#include <boost/multi_index/sequenced_index.hpp>
#include <boost/multi_index/random_access_index.hpp>
#include <boost/multi_index/ordered_index.hpp>
#include <boost/multi_index/hashed_index.hpp>
#include <boost/multi_index/identity.hpp>
#include <boost/multi_index/member.hpp>

#include "body.h"

using namespace ::boost::multi_index;

namespace physics {
	
	// Used as names for the multi-index magic
	struct by_body_0 {};
	struct by_body_1 {};
	struct by_contact {};
	struct by_velocity {};
	struct by_penetration {};
	
	template <typename d>
	struct MultiIndexContainer {
		typedef physics::Contact<d> Contact;
		typedef physics::Body<d> Body;
		
		// Boost multi-index magic
		typedef multi_index_container<
			Contact*,
			indexed_by<
				MULTI_INDEX_CONTACT_LIST_DEFAULT_INDEX<> // vector-like index
#if MULTI_INDEX_CONTACT_LIST_SORTED
				, ordered_non_unique< tag<by_velocity>, identity<Contact*>, CompareSpeed<Contact> > // by velocity
				, ordered_non_unique< tag<by_penetration>, identity<Contact*>, ComparePenetration<Contact> > // by penetration
				//, ordered_non_unique< tag<by_velocity>, member<Contact, float, &Contact::approachSpeed>, std::greater<float> > // by velocity
				//, ordered_non_unique< tag<by_penetration>, member<Contact, float, &Contact::penetration>, std::greater<float> > // by penetration
#endif
#if ! BODY_STORES_CONTACTS
				, MULTI_INDEX_CONTACT_LIST_LOOKUP< tag<by_body_0>, member<Contact, Body*, &Contact::body0> > // by body 0
				, MULTI_INDEX_CONTACT_LIST_LOOKUP< tag<by_body_1>, member<Contact, Body*, &Contact::body1> > // by body 1
#elif MULTI_INDEX_CONTACT_LIST_SORTED
				, MULTI_INDEX_CONTACT_LIST_LOOKUP< tag<by_contact>, identity<Contact*> > // by contact
#endif
			>
		> container;
	};
	
	template <typename d>
	class MultiIndexContactList : public BaseContactList<d, typename MultiIndexContainer<d>::container> {
		
		public:
		
		typedef physics::Contact<d> Contact;

		typedef BaseContactList<d, typename MultiIndexContainer<d>::container> Super;
		
		typedef typename Super::container container;
		typedef typename container::iterator iterator;
		
		virtual void prepareVelocities ();
		virtual void preparePenetrations ();
		
#if MULTI_INDEX_CONTACT_LIST_SORTED
		virtual Contact* getGreatestVelocity () const;
		virtual Contact* getGreatestPenetration () const;
#endif
		
		virtual void updateVelocities (Contact* data);
		virtual void updatePenetrations (Contact* data, float* moveLinear);
	};
};

#endif

