#ifndef _PHYSICS_DIMENSIONS_
#define _PHYSICS_DIMENSIONS_

namespace physics {
	class Globals;

	class Area;
	class Vector2D;
	class Orientation2D;
	class Rotation2D;
	class MOI2D;
	class Matrix2;
	class QuadtreeBroadPhase;
	
	class Volume;
	class Vector3D;
	class Orientation3D;
	class Rotation3D;
	class MOI3D;
	class Matrix3;
	class OctreeBroadPhase;
	
	template <typename d> class Body;
	template <typename d> class World;
	template <typename d> class Joint;
	template <typename d> class Contact;
	template <typename d> class ContactList;
	template <typename d> class CollisionGenerator;
	template <typename d> class ContactGenerator;
	template <typename d> class ContactResolver;
	
	struct TwoDee
	{
		typedef TwoDee Dimension;
		typedef physics::Body<TwoDee> Body;
		typedef physics::Joint<TwoDee> Joint;
		typedef physics::World<TwoDee> World;
		typedef physics::Contact<TwoDee> Contact;
		typedef physics::Area Shape;
		typedef physics::Vector2D Vector;
		typedef physics::Orientation2D Orientation;
		typedef physics::Rotation2D Rotation;
		typedef physics::MOI2D MOI;
		typedef physics::Matrix2 Matrix;
		typedef physics::QuadtreeBroadPhase HackBroadPhase;
	};

	struct ThreeDee
	{
		typedef ThreeDee Dimension;
		typedef physics::Body<ThreeDee> Body;
		typedef physics::Joint<ThreeDee> Joint;
		typedef physics::World<ThreeDee> World;
		typedef physics::Contact<ThreeDee> Contact;
		typedef physics::Volume Shape;
		typedef physics::Vector3D Vector;
		typedef physics::Orientation3D Orientation;
		typedef physics::Rotation3D Rotation;
		typedef physics::MOI3D MOI;
		typedef physics::Matrix3 Matrix;
		typedef physics::OctreeBroadPhase HackBroadPhase;
	};
	
	template <typename T>
	struct get_dimension {
		typedef typename T::Dimension Dimension;
	};
	
	template <typename T>
	struct get_dimension <T*> {
		typedef typename T::Dimension Dimension;
	};
	
	typedef Body<TwoDee> Body2D;
	typedef Body<ThreeDee> Body3D;
	
	typedef World<TwoDee> World2D;
	typedef World<ThreeDee> World3D;
	
	typedef Area Shape2D;
	typedef Volume Shape3D;
	
};

/*
You don't need to know what's going on here, even though it's simpler than it looks.
The useful macros are:

USE_DIMENSION(d): imports all type definitions from d
BODY_USE_DIMENSION(d): imports all type definitions from d other than Body
USE_2D(): imports type definitions from TwoDee
USE_3D(): imports type definitions from ThreeDee

*/

#define DIMENSION_TYPEDEF(d, Class) typedef d::Class Class

#define REAL_USE_DIMENSION_WITHOUT_BODY(d) \
	DIMENSION_TYPEDEF(d, Dimension); \
	DIMENSION_TYPEDEF(d, World); \
	DIMENSION_TYPEDEF(d, Shape); \
	DIMENSION_TYPEDEF(d, Vector); \
	DIMENSION_TYPEDEF(d, Orientation); \
	DIMENSION_TYPEDEF(d, Rotation); \
	DIMENSION_TYPEDEF(d, MOI); \
	DIMENSION_TYPEDEF(d, Matrix); \
	

#define REAL_USE_DIMENSION(d) \
	DIMENSION_TYPEDEF(d, Body); \
	REAL_USE_DIMENSION_WITHOUT_BODY(d)

#define USE_DIMENSION(d) REAL_USE_DIMENSION(typename d)

#define BODY_USE_DIMENSION(d) REAL_USE_DIMENSION_WITHOUT_BODY(typename d)

#define USE_2D() REAL_USE_DIMENSION(physics::TwoDee)

#define USE_3D() REAL_USE_DIMENSION(physics::ThreeDee)

/*
Implementation notes (only if you're curious):

We need a REAL version because if d is a template parameter, then
you need to tell the compiler that d::Vector (for example) is a type.
You do this by writing "typename d::Vector" instead of "d::Vector".
But if d is not a template parameter, you may _not_ write "typename".
See: http://pages.cs.wisc.edu/~driscoll/typename.html

We need a BODY version because the Body class cannot contain a type definition
for "Body". I.e. this will not compile:

class Body {
	typedef Foo Body;
	
	Body* pointer; // Is this a Body or a Foo?
};

*/

#endif

