#ifndef _PHYSICS_GLOBALS_
#define _PHYSICS_GLOBALS_

/***********************************************************************************\
This defines the Globals class, which holds lots of static data that can be modified
to adjust the behaviour of the library.
\***********************************************************************************/

namespace physics {
	/*! 
	*	The Globals class holds static data that can be modified to adjust the behaviour of the library
	*/
	class Globals {
		public:
		static float inverseTimeStep;

		static bool useSleeping;
		static float linearVelocitySleepEpsilon;
		static float angularVelocitySleepEpsilon;
		static float penetrationEpsilon;
		static float sleepRWABias;
		
		//drag parameters
		static bool airResistance;
		static bool linearDrag;
		static float linearDragCoefficient;
		static float quadraticDragCoefficient;
		
		static bool useRestitution;
		static float restitutionThreshold;

		static bool splitImpulses;
		
		static bool findMultipleContacts;
		static bool cacheContacts;
		
		static bool findSleepingContacts;
		
		// Recommended: MOVE_AFTER_COLLISIONS
		enum WhenToMoveEnum { MOVE_BEFORE_COLLISIONS, MOVE_AFTER_COLLISIONS, MOVE_AFTER_DRAWING };
		
		static WhenToMoveEnum whenToMove;
		
		static float _NEAR; // objects this far apart or less are considered near
		
		/**
		 * Default density for newly created shapes.
		 */
		static float DEFAULT_DENSITY;
		
		/**
		 * Default coefficient of restitution for newly created shapes.
		 */
		static float DEFAULT_RESTITUTION;
		
		/**
		 * Default coefficient of friction for newly created shapes.
		 */
		static float DEFAULT_FRICTION;
		
		/**
		 * Default value of gravity (in y axis).
		 */
		static float GRAVITY;
		
		/**
		 * Default length of a simulation step.
		 */
		static float DEFAULT_TIMESTEP;
		
		/**
		 * Mathematical constant pi.
		 */
		static const float PI;
				
	};
};

// Macro definitions

#define BODY_STORES_CONTACTS 1


#endif

