#ifndef _PHYSICS_JOINT_
#define _PHYSICS_JOINT_

#include "common.h"


#include "vector.h"
#include "orientation.h"
#include "rotation.h"
#include "moi.h"
#include "body.h"
#include "contact.h"

namespace physics {
	
	template <typename d>
	/*!
	*	Defines a joint that can be used in 2D and 3D simulations
	*/
	class Joint
	{
	public:

		USE_DIMENSION(d);
		
		//! Constructor
		Joint() : biasFactor(0.2f), localAnchor1(), localAnchor2(), P(), r1(), r2(), bias(), velocityPerImpulse(), softness(0.0f), ignoreContacts(true) {}

		void set(Body* body1, Body* body2, const Vector& anchor);

		void prepare();
		void applyImpulses();
		
		//! Checks if the joint has had its conditions violated
		/*! \return Boolean; True if the joint has been violated, false otherwise
		*/
		bool jointViolated();
		
		float biasFactor;

		Vector localAnchor1, localAnchor2;
		//! Accumulate the impulse here
		Vector P;
	
		// body objects involved in the joint
		Body* body1;
		Body* body2;

		Vector r1, r2, bias;

		Matrix velocityPerImpulse;

		float softness;
		
		bool ignoreContacts;

	private:

		


	};

}


#endif
