#ifndef _PHYSICS_MICROTIMER_
#define _PHYSICS_MICROTIMER_

#include "common.h"

#include <map>
#include <string>

#include <time.h>

#ifdef WIN32
	#include <winsock2.h>
	
	#ifndef VC_EXTRALEAN
	#define VC_EXTRALEAN
	#endif
	#ifndef WIN32_LEAN_AND_MEAN
	#define WIN32_LEAN_AND_MEAN
	#endif
	#ifndef WINDOWS_LEAN_AND_MEAN
	#define WINDOWS_LEAN_AND_MEAN
	#endif
	#define NOSERVICE
	#define NOMCX
	#define NOIME
	#define NONLS
	#undef min
	#undef max //No amount of preprocessor directives removes these from the windows
			   //include, so let's just undef them here. 
	#include <windows.h>
	struct timezone 
	{
		int  tz_minuteswest; /* minutes W of Greenwich */
		int  tz_dsttime;     /* type of dst correction */
	};
	
#else
#include <sys/time.h>
#endif /* ifdef WIN32 */


	


namespace physics {
	
	/*
	MicroTimer is similar to the Timer class in 2DSandbox, but it is accurate to the microsecond and is part of the physics namespace.
	It's used by the profiler currently, but I'm sure it has other potential uses.
	 - Ben
	*/
	class MicroTimer {
		public:
			
			MicroTimer ()
			{
				gettimeofday(&startTime, NULL);
			}
			
			~MicroTimer () {}
			
			void reset()
			{
				gettimeofday(&startTime, NULL);
			}
			
			double getTime() //Returns time since timer was last reset, in seconds as a double
			{
				timeval endTime;
				gettimeofday(&endTime, NULL);
	
				return (endTime.tv_sec + (endTime.tv_usec * 0.000001)) - (startTime.tv_sec + (startTime.tv_usec * 0.000001));
			}
			
			double getTime(std::string markerName) //Returns time since a given marker was set, in seconds as a double
			{
				timeval endTime;
				gettimeofday(&endTime, NULL);
	
				return (endTime.tv_sec + (endTime.tv_usec * 0.000001)) - (markerTimeMap[markerName].tv_sec + (markerTimeMap[markerName].tv_usec * 0.000001));
			}

			int getMicroTime() //Returns time since timer was last reset, in microseconds as an int
			{
				timeval endTime;
				gettimeofday(&endTime, NULL);
	
				return (endTime.tv_usec + (endTime.tv_sec * 1000000)) - (startTime.tv_usec + (startTime.tv_sec * 1000000));
			}
			
			int getMicroTime(std::string markerName) //Returns time since a given marker was set, in microseconds as an int
			{
				timeval endTime;
				gettimeofday(&endTime, NULL);
	
				return (endTime.tv_usec + (endTime.tv_sec * 1000000)) - (markerTimeMap[markerName].tv_usec + (markerTimeMap[markerName].tv_sec * 1000000));
			}
			
			void addMarker(std::string markerName)
			{
				gettimeofday(&markerTimeMap[markerName], NULL);
			}
			
		private:
			
			timeval startTime;
			std::map<const std::string, timeval> markerTimeMap;
		
#ifdef WIN32
			int gettimeofday(struct timeval *tv, struct timezone *tz);
#endif

	};
};

#endif
