#include "moi.h"

#include "body.h"
#include "shape.h"

#include <boost/foreach.hpp>

using namespace physics;

MOI2D::MOI2D (Body2D* body) {
	float value = 0;
	
	BOOST_FOREACH (Shape2D* shape, body->getShapes()) {
		value += shape->getMOI();
		
		float distance = shape->getLocalPosition().lengthSquared();
		value += distance * shape->getMass();
	}
	
	if (value < 0.0001f) {
		inverseMOI = 0;
	} else {
		inverseMOI = 1.0f / value;
	}
}

MOI3D::MOI3D (Body3D* body) : infinite(false) {
	Matrix3 moiTensor;
	
	BOOST_FOREACH (Shape3D* shape, body->getShapes()) {
		Orientation3D o = shape->getLocalOrientation();
		
		// Do I want Orientation * MOI * Orientation^-1
		// or do I want Orientation^-1 * MOI * Orientation?
		moiTensor += o.getMatrix() * Matrix3::diagonal(shape->getMOI()) * o.getInverseMatrix();
		
		Vector3D v = shape->getLocalPosition();
		
		// Hurrah! We agree with Wikipedia!
		moiTensor += shape->getMass() * (v.dot(v) * Matrix3::identity() - Matrix3::outerProduct(v, v));
	}
	
	localInverseInertiaTensor = moiTensor.getInverse();
	
	// Empty matrix
	if (localInverseInertiaTensor == Matrix3()) {
		infinite = true;
	}
	
	updateOrientation(body->getOrientation());
}



