#include "rect.h"
#include "polygon.h"

using namespace physics;

// Some day we will be able to profile the most asinine changes and see what effect they have...
#define RECT_AVOID_TRANSFORMS 0

Rect::Rect (float x1, float y1, float x2, float y2)
	: Area(Vector2D((x1 + x2) / 2, (y1 + y2) / 2)), halfWidth(x2 / 2 - x1 / 2), halfHeight(y2 / 2 - y1 / 2) { init(); }

Rect::Rect (const Vector2D& p1, const Vector2D& p2)
	: Area((p1 + p2) / 2), halfWidth(p2.x / 2 - p1.x / 2), halfHeight(p2.y / 2 - p1.y / 2) { init(); }

Rect::Rect (const Vector2D& p, float width, float height, const Orientation& o)
	: Area(p, o), halfWidth(width / 2), halfHeight(height / 2) { init(); }

bool Rect::contains (const Vector2D& worldP) const {
	Vector2D p = worldToLocal(worldP);
	
	return (p.x >= -halfWidth && p.x <= halfWidth && p.y >= -halfHeight && p.y <= halfHeight);
}

Vector2D Rect::getNormal (int i) const {
	#if RECT_AVOID_TRANSFORMS
	Vector2D dir = getVertex(i + 1) - getVertex(i);
	dir = dir.normalise();
	
	return Vector2D(dir.y, -dir.x);
	#else
	switch (i % 4) {
		case 0: return getOrientation().transform(Vector2D(-1, 0));
		case 1: return getOrientation().transform(Vector2D(0, -1));
		case 2: return getOrientation().transform(Vector2D(1, 0));
		case 3: return getOrientation().transform(Vector2D(0, 1));
	}
	
	return Vector2D();
	#endif
}

Vector2D Rect::getExtremePoint (const Vector2D& direction, int& i) const {
	Vector2D dir = getOrientation().inverseTransform(direction);
	
	if (dir.x < 0.0f) {
		i = (dir.y > 0.0f) ? 0 : 1;
	} else {
		i = (dir.y < 0.0f) ? 2 : 3;
	}
	
	return worldPoints[i];
}

void Rect::init () {
	if (halfWidth < 0) { halfWidth *= -1; }
	if (halfHeight < 0) { halfHeight *= -1; }
	
	thisMoved();
}

void Rect::thisMoved () {
	worldPoints[0] = localToWorld(Vector2D(-halfWidth, halfHeight));
	worldPoints[1] = localToWorld(Vector2D(-halfWidth, -halfHeight));
	worldPoints[2] = localToWorld(Vector2D(halfWidth, -halfHeight));
	worldPoints[3] = localToWorld(Vector2D(halfWidth, halfHeight));
}

bool Rect::intersects (const AARect& rect) const {
	//v1-----------v4
	//|            |
	//|            |
	//v2-----------v3
	
	//Does rect contains polygon centre
	if (rect.contains(Rect::getPosition())) {
		return 1;
	}

	//Does rect contain any Rect vertex
	for (int i = 0; i < Rect::getVertexCount(); i++) {
		if (rect.contains(Rect::getVertex(i))) {
			return 1;
		}
	}

	//Does polygon contain rect centre
	//TODO: Uncomment when AARect becomes a member of object class
	//if (Polygon::contains(rect.getPosition())) {
	//	return 1;
	//}

	//Does Rect contain any rect vertex
	for (int i = 0; i < 4; i++) {
		if (Rect::contains(rect.getVertex(i))) {
			return 1;
		}
	}

	return 0;
}
