#ifndef _PHYSICS_RECT_
#define _PHYSICS_RECT_

#include "common.h"

#include "area.h"

namespace physics {
	/*! 
	*	Provides the rectangle shape in the 2D simulation
	*/
	class Rect : public Area
	{
		public:
		Rect (float x1, float y1, float x2, float y2);
		Rect (const Vector2D& p1, const Vector2D& p2);
		Rect (const Vector2D& p, float width, float height, const Orientation& o = Orientation());
		
		float width () const { return halfWidth * 2; }
		float height () const { return halfHeight * 2; }
		
		void setWidth (float w) { halfWidth = w / 2; }
		void setHeight (float h) { halfHeight = h / 2; }
		
		Vector2D getSize () { return Vector2D(width(), height()); }
		void setSize (const Vector2D& size) { setWidth(size.x); setHeight(size.y); }
		void setSize (float w, float h) { setWidth(w); setHeight(h); }
		
		float getRadius () const { return 0; }
		
		bool contains (const Vector2D &p) const;
		bool intersects (const AARect& rect) const;
		
		Vector2D getNormal (int i) const;
		Vector2D getVertex (int i) const { return worldPoints[i % 4]; }
		
		int getVertexCount () const { return 4; }
		
		Vector2D getExtremePoint (const Vector2D& dir, int& i) const;
		
		float getArea () const { return width() * height(); }
		
		//! Get the Moment of Inertia
		/*! \return Float; Moment of Inertia
		*/
		float getMOI () const { return getMass() * (halfWidth * halfWidth + halfHeight * halfHeight) / 3; }
		
		Type type () const { return RECT; }
		
		private:
		
		void init ();
		
		virtual void thisMoved ();
		
		float halfWidth;
		float halfHeight;
		
		//v1-----------v4
		//|            |
		//|            |
		//v2-----------v3
		Vector2D worldPoints[4];
	};
};

#endif

