#ifndef _PHYSICS_SPHERE_
#define _PHYSICS_SPHERE_

#include "common.h"

#include "volume.h"

namespace physics {
	/*! \class Sphere
	*	Provides the sphere shape in the 3D simulation
	*/
	class Sphere : public Volume
	{
		public:
		
		Sphere () : m_radius(0) {}
		explicit Sphere (float r) : m_radius(r) {}
		Sphere (const Vector& p, float r) : Volume(p), m_radius(r) {}
		
		bool contains (const Vector &p) const;
		
		float getRadius () const { return m_radius; }
		void setRadius (float f) { m_radius = f; }
		
		Vector getNormal (int i) const { return Vector(); }
		Vector getVertex (int i) const { return getPosition(); }
		
		int getVertexCount () const { return 1; }
		int getEdgeCount () const { return 0; }
		int getFaceCount () const { return 1; }
		
		Vector getExtremePoint (const Vector& dir, int& i) const;
		
		virtual float getMinX () const { return getPosition().x - m_radius; }
		virtual float getMaxX () const { return getPosition().x + m_radius; }
		virtual float getMinY () const { return getPosition().y - m_radius; }
		virtual float getMaxY () const { return getPosition().y + m_radius; }
		virtual float getMinZ () const { return getPosition().z - m_radius; }
		virtual float getMaxZ () const { return getPosition().z + m_radius; }
		
		float getVolume () const { return 4.0f / 3.0f * Globals::PI * m_radius * m_radius * m_radius; }
		
		//! Get the Moment of Inertia
		/*! \return Vector3D; Moment of Inertia
		*/
		Vector3D getMOI () const; // Moment of Inertia
		
		Type type () const { return SPHERE; }
		
		private:
		float m_radius;
	};
};

inline bool physics::Sphere::contains (const physics::Vector3D &p) const {
	return (p.distanceSquared(getPosition()) <= getRadiusSq());
}

inline physics::Vector3D physics::Sphere::getExtremePoint (const physics::Vector3D& dir, int& i) const {
	i = 0;
	
	return getPosition() + dir * m_radius;
}

inline physics::Vector3D physics::Sphere::getMOI () const {
	float moi = 0.4f * getMass() * getRadiusSq();
	
	return Vector3D(moi, moi, moi);
}

#endif
