// Initial 3D sandbox app
// Currently does nothing but draw some lines and a cube - though a FPS style camera is implemented (albeit badly)
// should be fairly simple to extend this to give a 2.5D view of the current physics engine

#include <iostream>
#include <stdlib.h>

#include "SDL.h"
#include "SDL_opengl.h"

#include "camera.h"

using namespace physics;

//Screen attributes
const int SCREEN_WIDTH = 800;
const int SCREEN_HEIGHT = 600;
const int SCREEN_BPP = 32;

SDL_Event event;

Camera camera;
bool _isWireframe = false;

bool init()
{
    //Initialize SDL
    if( SDL_Init( SDL_INIT_EVERYTHING ) < 0 )
    {
        return false;    
    }
    
    
    //Create Window
    if( SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, SDL_OPENGL) == NULL )
    {
		return false;
    }
    
    //Set caption
    SDL_WM_SetCaption( "3D Sandbox - Torque To Me", NULL );
    
	
    return true;    
}

void clean_up()
{
 	
	//Quit SDL
    SDL_Quit();
}


void drawGrid(GLfloat size, GLint LinesX, GLint LinesZ)
{
	glBegin(GL_LINES);
	for (int xc = 0; xc < LinesX; xc++)
	{
		glVertex3f(	-size / 2.0 + xc / (GLfloat)(LinesX-1)*size, 0.0, size / 2.0);
		glVertex3f(	-size / 2.0 + xc / (GLfloat)(LinesX-1)*size, 0.0, size / -2.0);
	}
	for (int zc = 0; zc < LinesX; zc++)
	{
		glVertex3f(	size / 2.0, 0.0, -size / 2.0 + zc / (GLfloat)(LinesZ-1)*size);
		glVertex3f(	size / -2.0, 0.0, -size / 2.0 + zc / (GLfloat)(LinesZ-1)*size);
	}
	glEnd();
}



void drawCube(Vector3D position, float size)
{
	
	
	glTranslatef(position.x, position.y, position.z);
	glScalef(size,size,size);

	// start the stolen cube vertex definition!
	glBegin(GL_QUADS);									// Draw A Quad
		glColor3f(0.0f,0.5f,0.0f);						// Set The Color To Green
		glVertex3f( 0.5f, 0.5f,-0.5f);					// Top Right Of The Quad (Top)
		glVertex3f(-0.5f, 0.5f,-0.5f);					// Top Left Of The Quad (Top)
		glVertex3f(-0.5f, 0.5f, 0.5f);					// Bottom Left Of The Quad (Top)
		glVertex3f( 0.5f, 0.5f, 0.5f);					// Bottom Right Of The Quad (Top)
		
		glColor3f(0.5f,0.5f,0.0f);						// Set The Color To Orange
		glVertex3f( 0.5f,-0.5f, 0.5f);					// Top Right Of The Quad (Bottom)
		glVertex3f(-0.5f,-0.5f, 0.5f);					// Top Left Of The Quad (Bottom)
		glVertex3f(-0.5f,-0.5f,-0.5f);					// Bottom Left Of The Quad (Bottom)
		glVertex3f( 0.5f,-0.5f,-0.5f);					// Bottom Right Of The Quad (Bottom)
		glColor3f(0.5f,0.0f,0.0f);						// Set The Color To Red
		glVertex3f( 0.5f, 0.5f, 0.5f);					// Top Right Of The Quad (Front)
		glVertex3f(-0.5f, 0.5f, 0.5f);					// Top Left Of The Quad (Front)
		glVertex3f(-0.5f,-0.5f, 0.5f);					// Bottom Left Of The Quad (Front)
		glVertex3f( 0.5f,-0.5f, 0.5f);					// Bottom Right Of The Quad (Front)
		glColor3f(0.5f,0.5f,0.0f);						// Set The Color To Yellow
		glVertex3f( 0.5f,-0.5f,-0.5f);					// Top Right Of The Quad (Back)
		glVertex3f(-0.5f,-0.5f,-0.5f);					// Top Left Of The Quad (Back)
		glVertex3f(-0.5f, 0.5f,-0.5f);					// Bottom Left Of The Quad (Back)
		glVertex3f( 0.5f, 0.5f,-0.5f);					// Bottom Right Of The Quad (Back)
		glColor3f(0.0f,0.0f,0.5f);						// Set The Color To Blue
		glVertex3f(-0.5f, 0.5f, 0.5f);					// Top Right Of The Quad (Left)
		glVertex3f(-0.5f, 0.5f,-0.5f);					// Top Left Of The Quad (Left)
		glVertex3f(-0.5f,-0.5f,-0.5f);					// Bottom Left Of The Quad (Left)
		glVertex3f(-0.5f,-0.5f, 0.5f);					// Bottom Right Of The Quad (Left)
		glColor3f(0.5f,0.0f,0.5f);						// Set The Color To Violet
		glVertex3f( 0.5f, 0.5f,-0.5f);					// Top Right Of The Quad (Right)
		glVertex3f( 0.5f, 0.5f, 0.5f);					// Top Left Of The Quad (Right)
		glVertex3f( 0.5f,-0.5f, 0.5f);					// Bottom Left Of The Quad (Right)
		glVertex3f( 0.5f,-0.5f,-0.5f);					// Bottom Right Of The Quad (Right)
	glEnd();											// Done Drawing The Quad


}
void drawScene() {
		
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	
	glLoadIdentity();
			
	// set camera transform
	camera.render();
		

	// draw in black 
	glColor3f(0.0,0.0,0.0);

	// draw the floor grid
	glPushMatrix();
		drawGrid(10,40,40);
	glPopMatrix();

	// draw the cube
	glPushMatrix();
		drawCube(Vector3D(0,1.5,0), 3);
	glPopMatrix();
		
		
}     

int main( int argc, char *argv[] ){
    
	//Quit flag
    bool quit = false;
       
    bool mouseDown = false;
    bool fullscreen = false;
    //Initialize
    
	if( init() == false )
    {
        return 1;    
    }
    
    // Set up the viewport and perspective matrix
	glViewport(0,0,SCREEN_WIDTH,SCREEN_HEIGHT);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0f, SCREEN_WIDTH / SCREEN_HEIGHT, 1.0f, 500.0f);
    glMatrixMode(GL_MODELVIEW);
            

	glShadeModel(GL_SMOOTH);							
	glClearColor(1.0f, 1.0f, 1.0f, 0.5f);				
	glClearDepth(1.0f);									
	glEnable(GL_DEPTH_TEST);							
	glDepthFunc(GL_LEQUAL);								
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	

 
          
    camera.setCamera(Vector3D(50,50,0),Vector3D(0.0f,0.0f,-1.0f),Vector3D(0.0f,1.0f,0.0f));
    
      
	while( quit == false )
	{
        
        //grabs a pointer into the keyboard state array
        Uint8 * keystate = SDL_GetKeyState(NULL);
        
        if(keystate[SDLK_LEFT]){
        	camera.strafeCamera(0.1f);
        }
        
        if(keystate[SDLK_RIGHT]){
        	camera.strafeCamera(-0.1f);
        }
        
        if(keystate[SDLK_UP]){
          	camera.moveforwardCamera(-0.1f);
        }
        
        if(keystate[SDLK_DOWN]){
           	camera.moveforwardCamera(0.1f);
        }
        
        //While there are events to handle
		while( SDL_PollEvent( &event ) )
		{
         
			if (event.type == SDL_MOUSEBUTTONDOWN){
				
				switch(event.button.button){
				
				case SDL_BUTTON_RIGHT: 
					mouseDown = true; 
					//SDL_ShowCursor(0);  // disable drawing the cursor
					break;
				
				default: break;
				}
				
			}
			
			if (event.type == SDL_MOUSEBUTTONUP){
							
				switch(event.button.button){
							
				case SDL_BUTTON_RIGHT: 
					mouseDown = false; 
					//SDL_ShowCursor(1);  //enable the cursor again
				break;
				
				default: break;
				}
							
			}
			
			
			if (event.type == SDL_MOUSEMOTION && mouseDown){
				camera.testMouse(event.motion.xrel,event.motion.yrel);
			}
			
			if (event.type == SDL_KEYDOWN){
				
				switch(event.key.keysym.sym){
				
				case SDLK_ESCAPE: quit = true; break;
				
				case SDLK_w: _isWireframe = !_isWireframe; break;
			
				
				case SDLK_f:
					// quick fullscreen hack - no error checking is being done or anything... /slap
					
					fullscreen = !fullscreen;
					
					if (fullscreen){
						SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, SDL_OPENGL | SDL_FULLSCREEN);
					}
					else{
						SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, SDL_OPENGL);
					}
					
					break;
								
				default: break;
								
				}
								
			}
			
			if( event.type == SDL_QUIT )
			{
                quit = true;
            }
		}
	
	    drawScene();
	    	    
	    SDL_GL_SwapBuffers();  // swap buffer
	  
	}
	
	//Clean up
	clean_up();
	
	return 0;
}








