#include "camera2d.h"
#include "camera3d.h"
#include "body.h"

void Camera2D::copyState (const Camera3D* other) {
	centre = other->centre;
	
	float ratio = (float) Window::height() / (float) Window::width();
	
	scale = MAGIC_CONVERSION_CONSTANT * ratio / other->distance;
	
	targetScale = scale;
}

bool Camera2D::isVisible (physics::Body2D* body) const {
	const float r = body->getBoundingRadius();
	
	const physics::Vector2D p = body->getPosition();
	
	return (p.x + r >= minWorld.x && p.x - r <= maxWorld.x && p.y + r >= minWorld.y && p.y - r <= maxWorld.y);
}

physics::Vector3D Camera2D::screen2world (int x, int y) const {
	physics::Vector3D v;
	
	int width = Window::width();
	int height = Window::height();
	
	// Make it scale when you resize window or go fullscreen
	float realScale = scale / (640.0f / width);
	
	v.x = centre.x + (x - width / 2) / realScale;
	v.y = centre.y - (y - height / 2) / realScale;
	return v;
}

void Camera2D::update () {
	if (targetScale > 500) { targetScale = 500; }
	if (targetScale < 2.0f) { targetScale = 2.0f; }
	
	if (scale != targetScale) {
		float scaleChange = (targetScale - scale) / 10;
		scale += scaleChange;
		centre = mouse + (centre - mouse) * (scale - scaleChange) / scale;
	}
	
	const float minY = 0.0f;
	const float maxY = 100.0f;
	const float minX = -100.0f;
	const float maxX = 100.0f;
	
	if (centre.y < minY) { centre.y = minY; }
	if (centre.y > maxY) { centre.y = maxY; }
	if (centre.x < minX) { centre.x = minX; }
	if (centre.x > maxX) { centre.x = maxX; }
	
	// Record visible bounds of the camera for draw culling
	
	const int width = Window::width();
	const int height = Window::height();
	
	minWorld = screen2world(0, height);
	maxWorld = screen2world(width, 0);
}

void Camera2D::use () {
	int width = Window::width();
	int height = Window::height();
	
	// Make it scale when you resize window or go fullscreen
	height = (int) (height * 640.0f / width);
	width = 640;
	
	// Set up the projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glScalef(scale, scale, 1);
	glOrtho(-width/2, width/2, -height/2, height/2, 1, 400);
	glTranslatef(-centre.x, -centre.y, -2);
	
	// Make lines look better
	
	glEnable( GL_LINE_SMOOTH );
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	
	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glLineWidth (1.5);
}

