#ifndef _CAMERA_3D_
#define _CAMERA_3D_

#include "camera.h"
#include "window.h"
#include "frustum.h"

class Camera2D;

class Camera3D : public Camera {
	public:
	
	Camera3D(bool sandbox3D = false) : centre(5, 10), camera(5, 10, 35), distance(35), targetDistance(35), true3D(sandbox3D) {}
	
	// Returns position of mouse
	physics::Vector3D screen2world (int x, int y) const;
	
	physics::Vector3D getNear (int x, int y) const;
	physics::Vector3D getFar (int x, int y) const;
	
	physics::Vector3D getNormal (int x, int y) const { return (getFar(x, y) - getNear(x, y)).normalise(); }
	
	physics::Vector3D getPosition () const { return camera; }
	void setPosition (const physics::Vector3D& p) { camera = p; }

	void zoomIn (int x, int y) { targetDistance /= 1.2f; mouseScreen = physics::Vector2D(x, y); mouseWorld = screen2world(x, y); }
	void zoomOut (int x, int y) { targetDistance *= 1.2f; mouseScreen = physics::Vector2D(x, y); mouseWorld = screen2world(x, y); }
	
	void translate (const physics::Vector3D& t) { centre += t; camera += t; }
	
	void rotate (const physics::Vector2D& t);
	
	void copyState (const Camera2D* c);
	
	void update ();
	void use ();
	
	bool isVisible (physics::Body2D* body) const { return frustum.inside(body); }
	bool isVisible (physics::Body3D* body) const { return frustum.inside(body); }
	
	bool is2D() const { return false; }
	bool is3D() const { return true; }
	
	private:
	
	friend class Camera2D;
	
	physics::Vector3D centre;
	physics::Vector2D mouseScreen;
	physics::Vector3D mouseWorld;
	physics::Vector3D camera;
	float distance;
	float targetDistance;
	bool true3D;
	
	Frustum frustum;
	//float angle;
};

#endif

