#include "sandbox2d.h"
#include "sandbox3d.h"
#include "trapfpe.h"
#include "window.h"
#include "timer.h"
#include "drawing.h"

#include <stdlib.h> 
#include <iostream>

void init (int argc, char *argv[]) {
	physics::Profiler::profiler.enable(); //enable the profiler by default
	
	bool do3D = false;
	bool force2D = false;
	bool isInteractive = true;
	int scene = 0;
	int script = 0;
	std::string lastS;	

	for (int i = 0; i < argc; i++) {
		std::string s = argv[i];
		
		if (s == "3D" || s == "3d") {
			do3D = true;
		} else if (s == "2D" || s == "2d") {
			force2D = true;
		} else if (s == "-i") {
			isInteractive = false;
		} else if (s == "-p") {
			physics::Profiler::profiler.setFramesToStore(0); //unlimited logging
			physics::Profiler::profiler.enableFileOutput(); //enable file writing.
			physics::Profiler::profiler.enableStdOutput(); //enable output to stdout.
		} else if (s == "-g") {
			physics::Profiler::profiler.setFramesToStore(0); //unlimited logging
			physics::Profiler::profiler.enableFileOutput(); //enable file writing.
			physics::Profiler::profiler.enableStdOutput(); //enable output to stdout.
			physics::Profiler::profiler.enableGraphOutput(); //enable graph plotting.
		} else if (s == "-s" || s == "-o" || s == "-S") {
			//recognise options that need parameters...
		} else if (lastS == "-o") {
			physics::Profiler::profiler.setOutputFilename(s);
		} else if (lastS == "-s") {
			scene = std::atoi(s.c_str());
			if (scene < 0 || scene > 10000) {
				std::cout << "invalid scene: '" << s << "'. Must be a number from 0-9." << std::endl;
				scene = 0; //if it's invalid, warn and reset to default.
			} 
		} else if (lastS == "-S") {
			script = std::atoi(s.c_str());
			if (script < 0 || script > 2) {
				std::cout << "invalid script: '" << s << "'. Must be a number from 0-2." << std::endl;
				script = 0; //if it's invalid, warn and reset to default.
			} 
		} else if (s == "--help") {
			std::cout << "Large Polygon Collider Sandbox" << std::endl;
			std::cout << "Usage: sandbox [2d|3d] [-p | -g] [-i] [-o outputfile] [-s scene] [-S script]" << std::endl;
			exit(0);
		} else if (s == "--no-sleeping") {
			physics::Globals::useSleeping = false;
		} else if (i != 0) { //If we haven't recognised the arg yet, and the arg isn't the binary itself...
			std::cout << "Unrecognised option: " << s << std::endl;
			std::cout << "Use --help for usage info." << std::endl;
			exit(1);
		}
		
		lastS = s;
	}
	
	if (! Window::init(640, 480, "Large Polygon Collider - Sandbox", true)) {
		exit(1);
	}
	
	Drawing::init();
	
	if (do3D) {
		Controller::active = new Sandbox3D();
	} else {
		Controller::active = new Sandbox2D(scene, script, force2D, isInteractive);
	}
}

void update () {
	Controller::active->update();
}

void draw () {
	Window::beginDraw();
	
	Controller::active->draw();
	
	Window::endDraw();
}

int main(int argc, char *argv[]) {
	init(argc, argv);
	
	const float timestep = 1.0f / 60.0f;
	
	Timer timer;
	double oldTime = timer.getTime();
	double remainder = 0.0;
	
	while (true) {
		trapfpe(); // Trap floating point exceptions
		
		Input::checkEvents();
		
		update();
		
		notrapfpe();
		
		draw();
		
		remainder = timer.getTime() - oldTime;

		// our timers (and SDL_Delay) are not really accurate enough to do this yet, but the spinlocking
		// was making my laptop cry.

		if (remainder < timestep) {
			SDL_Delay( (int) ((timestep - remainder) * 1000));
		}
		oldTime = timer.getTime();		
	}
	
	physics::Profiler::profiler.end(); //write profiler output on quit.
	return 0;
}
