#ifndef _SANDBOX_2D_
#define _SANDBOX_2D_

#include "input.h"
#include "director.h"
#include "common.h"
#include "vector.h"
#include <vector>

class Camera;
class Camera2D;
class Camera3D;

class Sandbox2D : public Controller {
	public:
	
	Sandbox2D (int scene = 0, int script = 0, bool force2D = false, bool interactive = true);
	virtual ~Sandbox2D () {}
	
	virtual void update ();
	virtual void draw () const;
	
	virtual void keyDown (SDL_Event& event);
	virtual void keyUp (SDL_Event& event);
	virtual void mouseDown (SDL_Event& event);
	virtual void mouseUp (SDL_Event& event);

	//void DisplayKey(SDL_Event& event);

	void reset();
	void clearWorld();
	void killWorld();
	void togglePause();
	void toggleCamera();

	void loadPreset(int p);
	int getActiveScene();
	void setActiveDirectorScene(int activeScene);
	
	// Member data
	private:

	physics::World2D* world;
	physics::Vector2D mouse;
	
	Director* director;

	physics::Vector2D clickPos;
	
	bool drawContacts;
	
	//physics::Body2D* rotateObject;
	physics::Body2D* orientateObject;
	physics::Body2D* dragObject;
	physics::Vector2D dragOffset;
	
	bool paused;
	bool panning;
	enum State {IDLE, LOADING, SELECTING, CARRYING, ORIENTATING, CIRCLE, LINE, CAPSULE, RECT, POLYGON, JOINT, SPRING};
	//Idle - Default state
	//Loading - Drop-down console has focus
	//Selecting - Left Mouse is down and a selection box is being drawn
	//Carrying - Left Mouse is down and a polygon is being carried, true in both pause and unpaused states
	//Orientating - Spinning a body using mousewheel
	//Circle - C is down, circle preview being made, circle about to be drawn
	//Line - Unused?
	//Capsule - Unused
	//Rect - B is down, box preview being made, box about to be drawn
	//Polygon - Ctrl+click has been pressed. Control is down. When control is up polygon about to be drawn. 
	//ROTATING, <-- setting rate of orientation, "spin". May add later. 
	//Joint - Creating joint
	State interfaceState;

	int activeScene;
	int activeScript;
	int verbosity;
	
	bool interactiveMode;
	
	Camera* camera;
	Camera2D* camera2D;
	Camera3D* camera3D;


	//stores clicked-points in arbitrary polygon creation (Ctrl+click)
	std::list<physics::Vector2D> vertexList;

	//stores copied bodies
	std::list<physics::Body2D*> clipboard;

};

#endif
