#ifndef _TRAPFPE_H_
#define _TRAPFPE_H_


// Generates run-time exceptions when NaN is produced, on division by zero,
// and on floating point overflow

#ifdef _MSC_VER
#include <float.h>

inline void trapfpe() {
  //Blargh
  
  //Set the x86 floating-point control word according to what
	//exceptions you want to trap. 
	_clearfp(); //Always call _clearfp before setting the control
	            //word
	//Because the second parameter in the following call is 0, it
	//only returns the floating-point control word
	unsigned int cw = _controlfp(0, 0); //Get the default control
	                                    //word
	//Set the exception masks off for exceptions that you want to
	//trap.  When a mask bit is set, the corresponding floating-point
	//exception is //blocked from being generating.
	cw &=~(EM_OVERFLOW|EM_ZERODIVIDE|EM_INVALID);
	//For any bit in the second parameter (mask) that is 1, the 
	//corresponding bit in the first parameter is used to update
	//the control word.  
	unsigned int cwOriginal = _controlfp(cw, MCW_EM); //Set it.
								//MCW_EM is defined in float.h.
								//Restore the original value when done:
								//_controlfp(cwOriginal, MCW_EM);

	unsigned long cntrReg;
	_asm
	{
		stmxcsr [cntrReg]        	//Get MXCSR register
		and [cntrReg], 0FFFFFF7Fh	//bit 7 - invalid instruction mask
							        //bit 9 - divide-by-zero mask
							        //bit 10 - overflow mask
							        //bit 11 - underflow mask	 
		ldmxcsr [cntrReg]			//Load MXCSR register
	}
}

inline void notrapfpe() {}

#elif ! defined(WIN32) && ! defined(DARWIN)

#include <fenv.h>

inline void trapfpe() {
  feenableexcept(FE_INVALID|FE_DIVBYZERO|FE_OVERFLOW);
}

inline void notrapfpe() {
  fedisableexcept(FE_INVALID|FE_DIVBYZERO|FE_OVERFLOW);
}

#else
inline void trapfpe() {}
inline void notrapfpe() {}
#endif

#endif /* _TRAPFPE_H_ */

