#include "window.h"

#include <cstdlib> // For some useful functions such as atexit :)
#include <iostream>

#include "drawing.h"

using namespace std;

bool Window::s_ready = false;

int Window::s_width = 800;
int Window::s_height = 600;

int Window::s_fullscreen_width;
int Window::s_fullscreen_height;

int Window::s_old_width;
int Window::s_old_height;

bool Window::s_resizable = false;
bool Window::s_fullscreen = false;

SDL_Surface* Window::s_screen;
FTFont* Window::guiFont;

bool Window::init (const int w, const int h, const char* title, bool resizable) {
	s_width = w;
	s_height = h;

	if ( SDL_Init(SDL_INIT_VIDEO) != 0 ) {
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
		return false;
	}

	atexit(SDL_Quit); // Clean it up nicely :)
	
	const SDL_VideoInfo* videoInfo = SDL_GetVideoInfo();
	
	s_fullscreen_width = videoInfo->current_w;
	s_fullscreen_height = videoInfo->current_h;
	
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	
	s_ready = true;
	
	if (title) { setTitle(title); }
	
	s_resizable = resizable;

	return reset();
}

bool Window::reset () {
	
	if (! s_ready) {
		return false;
	}
	
	Uint32 flags = SDL_OPENGL | SDL_ANYFORMAT;
	
	if (s_resizable) {flags |= SDL_RESIZABLE; }
	if (s_fullscreen) {flags |= SDL_FULLSCREEN; }
	
	s_screen = SDL_SetVideoMode(s_width, s_height, 32, flags);
	
	if (s_screen == NULL) {
		fprintf(stderr, "Unable to set video mode: %s\n", SDL_GetError());
		return false;
	}
	
	glViewport(0, 0, s_width, s_height);
	
	Drawing::init();
	
#ifdef USE_FTGL
	// reset the font object
	if (guiFont){
		if(guiFont->Error()) {
			delete guiFont;
			guiFont = new FTGLTextureFont("data/Courier_New.ttf");
		}
	}
	else
	{
		guiFont = new FTGLTextureFont("data/Courier_New.ttf");
	}

	if(guiFont->Error()) {
		std::cerr << "Can't load font!\n";
		return false;	// no more silent font loading failings 
	} else {
		guiFont->FaceSize(16);
	}
#endif

	return true;
}

bool Window::setTitle (const char* title) {
	if (! s_ready) { return false; }
	
	SDL_WM_SetCaption(title, title);
	
	return true;
}

void Window::setWidth (const int w) {
	s_width = w;
	
	reset();
}

void Window::setHeight (const int h) {
	s_height = h;
	
	reset();
}

void Window::resize (const int w, const int h) {
	s_width = w;
	s_height = h;
	
	reset();
}

void Window::setFullscreen (bool b) {
	s_fullscreen = b;
	
	if (s_fullscreen) {
		s_old_width = s_width;
		s_old_height = s_height;
		
		if (s_resizable) {
			s_width = s_fullscreen_width;
			s_height = s_fullscreen_height;
		}
	} else if (s_resizable) {
		s_width = s_old_width;
		s_height = s_old_height;
	}
	
	reset();
}

void Window::toggleFullscreen () {
	setFullscreen(! s_fullscreen);
}

void Window::beginDraw () {
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
}

void Window::endDraw () {
	SDL_GL_SwapBuffers();
}

void Window::drawText(const char *text)
{
#ifdef USE_FTGL
	if (guiFont)
		if (!guiFont->Error())
			
			guiFont->Render(text);
#endif
}

